package elements;

import elements.exceptions.UserNotExistingException;

import java.time.LocalDate;
import java.util.*;

/**
 * Created by vylion on 4/21/16.
 */
public class Chat {
    private static final String DATE_TAG = "Date=";

    private Map<String, Integer> users;
    private LocalDate lastPole;

    public Chat() {
        users = new HashMap<String, Integer>();
        lastPole = null;
    }

    public boolean exists(String user) {
        return users.containsKey(user);
    }

    public void add(String user) {
        users.put(user, 0);
    }

    public void add(String user, int p) {
        users.put(user, p);
    }

    public void winPole(String user) {
        if(!users.containsKey(user)) add(user);
        users.replace(user, users.get(user)+1);
        lastPole = LocalDate.now();
    }

    public boolean availablePole() {
        LocalDate today = LocalDate.now();

        return (lastPole == null || today.isAfter(lastPole));
    }

    public String getRanking() {
        List<Map.Entry<String, Integer>> top;
        String ranking = "";
        Map.Entry<String, Integer>[] ts = new Map.Entry[users.size()];

        top = Arrays.asList((users.entrySet().toArray(ts)));
        top.sort(new UserComparator());

        for(int i = 0; i < top.size(); i++) {
            ranking += "[" + (i+1) + ".] @" + getUsername(top.get(i)) + ": ";
            ranking += getPoles(top.get(i)) + "\n";
        }

        return ranking;
    }

    public String toTxt() {
        String s = DATE_TAG;
        if(lastPole == null) s += "null\n";
        else s += lastPole + "\n";
        Iterator it = users.entrySet().iterator();

        while(it.hasNext()) {
            Map.Entry<String, Integer> entry = (Map.Entry<String, Integer>) it.next();
            s += getUsername(entry) + ":" + getPoles(entry) + "\n";
        }

        return s;
    }

    public static Chat fromTxt(List<String> lines) {
        Chat c = new Chat();

        if(lines.get(0).startsWith(DATE_TAG)) {
            String pole = lines.get(0).substring(DATE_TAG.length());
            String[] reading = pole.split("-");
            LocalDate poleDate = LocalDate.of(Integer.parseInt(reading[0]),
                    Integer.parseInt(reading[1]), Integer.parseInt(reading[2]));
            c.setPole(poleDate);

            for(int i = 1; i < lines.size(); i++) {
                reading = lines.get(i).split(":");
                c.add(reading[0],Integer.parseInt(reading[1]));
            }
        }

        else {
            for(int i = 0; i < lines.size(); i++) {
                String[] reading = lines.get(i).split(":");
                c.add(reading[0],Integer.parseInt(reading[1]));
            }
        }


        return c;
    }

    private void setPole(LocalDate d) {
        lastPole = d;
    }

    private String getUsername(Map.Entry<String, Integer> e) {
        return e.getKey();
    }

    private int getPoles(Map.Entry<String, Integer> e) {
        return e.getValue();
    }
}

class UserComparator implements Comparator<Map.Entry> {

    @Override
    public int compare(Map.Entry entr1, Map.Entry entr2) {

        return ((Integer) entr2.getValue()).compareTo((Integer) entr1.getValue());
    }
}
