package main;

import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.JsonNode;
import com.mashape.unirest.http.Unirest;
import com.mashape.unirest.http.exceptions.UnirestException;
import elements.Chat;
import elements.Persistence;
import elements.exceptions.PoleBlockedException;
import elements.exceptions.ReadErrorException;
import elements.exceptions.SaveErrorException;
import org.json.JSONArray;
import org.json.JSONObject;

import java.util.*;

/**
 * Created by vylion on 4/1/16.
 */
public class Vylbot {
    private static final String TOKEN = "165232232:AAGVoVm1AA_cP2RNGh3sR4nPX9hQvujr_ls";
    private static final String LOGTAG = "VYLBOT";
    private static final String USERNAME = "VylionBot";
    private static final String BASE_URL = "https://api.telegram.org/bot" + TOKEN;
    private static final String PARSE_MARKDOWN = "Markdown";
    private static final String PARSE_HTML = "HTML";

    private static final int COMMAND_COUNT_START = 0;
    private static final int COMMAND_START = COMMAND_COUNT_START;
    private static final int COMMAND_HELP = COMMAND_START + 1;
    private static final int COMMAND_RANKING = COMMAND_HELP + 1;
    private static final int COMMAND_TALIBOT_STATS = COMMAND_RANKING + 1;
    private static final int COMMAND_COUNT_END = COMMAND_TALIBOT_STATS + 1;

    private static final int WARNING_COUNT_START = COMMAND_COUNT_END;
    private static final int WARNING_UNKNOWN_ERROR = WARNING_COUNT_START;
    private static final int WARNING_NO_USERNAME = WARNING_UNKNOWN_ERROR + 1;
    private static final int WARNING_POLE_BLOCKED = WARNING_NO_USERNAME + 1;
    private static final int WARNING_NOT_IMPLEMENTED = WARNING_POLE_BLOCKED + 1;
    private static final int WARNING_NO_GROUP = WARNING_NOT_IMPLEMENTED + 1;
    private static final int WARNING_COUNT_END = WARNING_NO_GROUP + 1;

    private List<String> commands;
    private Map<Long, Chat> chats;

    public Vylbot() throws ReadErrorException {
        chats = Persistence.getInstance().readChats();
        commands = new ArrayList<>();

        commands.add(COMMAND_START, "/start");
        commands.add(COMMAND_HELP, "/help");
        commands.add(COMMAND_RANKING, "/ranking");
        commands.add(COMMAND_TALIBOT_STATS, "!pole stats");
    }

    public String getUsername() {
        return USERNAME;
    }

    public String getToken() {
        return TOKEN;
    }

    public String myName() {
        return "Vylbot";
    }

    //--- Http handling

    public HttpResponse<JsonNode> sendMessage(Long chatId, String text) throws UnirestException {
        System.out.println("Sending message: \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", chatId)
                .field("text", text)
                .asJson();
    }

    public HttpResponse<JsonNode> sendMessage(Long chatId, String text, String parseMode) throws UnirestException {
        System.out.println("Sending message: \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", chatId)
                .field("text", text)
                .field("parse_mode", parseMode)
                .asJson();
    }

    public HttpResponse<JsonNode> sendMessage(String channel, String text) throws UnirestException {
        System.out.println("Sending message to " + channel + ": \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", channel)
                .field("text", text)
                .asJson();
    }

    public HttpResponse<JsonNode> sendMessage(String channel, String text, String parseMode) throws UnirestException {
        System.out.println("Sending message to " + channel + ": \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", channel)
                .field("text", text)
                .field("parse_mode", parseMode)
                .asJson();
    }

    public HttpResponse<JsonNode> replyMessage(Long chatId, Integer repliedMessage, String text) throws UnirestException {
        System.out.println("Sending message: \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", chatId)
                .field("text", text)
                .field("reply_to_message_id", repliedMessage)
                .asJson();
    }

    public HttpResponse<JsonNode> getUpdates(Integer offset) throws UnirestException {
        return Unirest.post(BASE_URL + "/getUpdates")
                .field("offset", offset)
                .asJson();
    }

    public HttpResponse<JsonNode> getUpdates(Integer offset, Integer timeout) throws UnirestException {
        return Unirest.post(BASE_URL + "/getUpdates")
                .field("offset", offset)
                .field("timeout", timeout)
                .asJson();
    }

    //--- bot thinking

    public void run() throws UnirestException {
        int last_upd_id = 0;
        HttpResponse<JsonNode> response;
        System.out.println("Listening.\n");

        while (true) {
            response = getUpdates(last_upd_id++);
            if (response.getStatus() == 200) {
                JSONArray responses = response.getBody().getObject().getJSONArray("result");
                if (responses.isNull(0)) continue;
                else last_upd_id = responses.getJSONObject(responses.length() - 1).getInt("update_id") + 1;

                for (int i = 0; i < responses.length(); i++) {
                    JSONObject message = responses.getJSONObject(i).getJSONObject("message");

                    processMessage(message);
                }
            }
        }
    }

    private void processMessage(JSONObject message) throws UnirestException {
        //Get chat id, the most important thing
        long chat_id = message.getJSONObject("chat").getLong("id");

        //Get basic compulsory variables
        int message_id = message.getInt("message_id");
        String chatType = message.getJSONObject("chat").getString("type");
        String name = message.getJSONObject("from").getString("first_name");

        //Declare basic optional variables
        String user = null;

        //Check for user last name
        if (message.getJSONObject("from").has("last_name"))
            name += " " + message.getJSONObject("from").getString("last_name");

        //Check for user username
        if (message.getJSONObject("from").has("username"))
            user = message.getJSONObject("from").getString("username");

        //Console log
        boolean full_log = true;
        if (full_log) System.out.println("\n" + message);
        else {
            if (message.getJSONObject("chat").has("title"))
                System.out.println("\nMessage received from chat " +
                        message.getJSONObject("chat").getString("title") +
                        " (chat id: " + chat_id + ")");
            else if (user != null)
                System.out.println("\nMessage received from user @" + user +
                        " (chat id: " + chat_id + ")");
            else
                System.out.println("\nMessage received from user " + name +
                        " (chat id: " + chat_id + ")");
        }

        //Actually read the message
        if (message.has("text")) {
            String text = message.getString("text");

            if (text.startsWith("/") || text.startsWith("!")) handleInput(chat_id, chatType.contains("group"), user, text);
            else handleText(chat_id, chatType.contains("group"), user, text);

            return;
        }
    }

    //--- Command handling

    private void handleInput(long chat_id, boolean group, String user, String text) throws UnirestException {

        for(int i = COMMAND_COUNT_START; i < COMMAND_COUNT_END; i++) {
            if (text.startsWith(commands.get(i))) {
                if(text.startsWith(commands.get(i) + "@") && !text.startsWith(commands.get(i) + "@" + USERNAME)) return;

                else {
                    if(group) handleCommand(chat_id, i, user, text);
                    else handlePrivateCommand(chat_id, i, user, text);
                    return;
                }
            }
        }
    }

    private void handleCommand(long chat_id, int command, String user, String text) throws UnirestException {
        switch (command) {
            case COMMAND_START:
                sendMessage(chat_id, getInformation(COMMAND_START));
                return;

            case COMMAND_HELP:
                sendMessage(chat_id, getInformation(COMMAND_HELP), PARSE_MARKDOWN);
                return;

            case COMMAND_TALIBOT_STATS:
            case COMMAND_RANKING:
                sendMessage(chat_id, chats.get(chat_id).getRanking());
                return;

            default: return;
        }
    }

    private void handlePrivateCommand(long chat_id, int command, String user, String text) throws UnirestException {
        switch (command) {
            case COMMAND_START:
                sendMessage(chat_id, getInformation(COMMAND_START));
                return;

            case COMMAND_HELP:
                sendMessage(chat_id, getInformation(COMMAND_HELP));
                return;

            default:
                return;
        }
    }

    private void handleText(long chat_id, boolean group, String user, String text) throws UnirestException {
        if(text.toLowerCase().contains("pole")) {
            if(group) {
                if (user == null) sendMessage(chat_id, getInformation(WARNING_NO_USERNAME));
                else try {
                    sendMessage(chat_id, winPole(chat_id, user));
                } catch (PoleBlockedException e) {
                    return;
                }
            }
            else sendMessage(chat_id, getInformation(WARNING_NO_GROUP));
        }
    }

    private boolean exists(long chat_id, String user) {
        return  chats.containsKey(chat_id) && chats.get(chat_id).exists(user);
    }

    private String winPole(long chat_id, String user) throws PoleBlockedException {
        if(!chats.containsKey(chat_id)) chats.put(chat_id, new Chat());
        if(!chats.get(chat_id).availablePole()) throw new PoleBlockedException();

        try {
            chats.get(chat_id).winPole(user);
            saveChat(chat_id);
            return "El usuario @" + user + " ha hecho la pole!";
        } catch (SaveErrorException e) {
            return e.getMessage();
        }
    }

    private void saveChat(long chat_id) throws SaveErrorException {
        try {
            Persistence.getInstance().saveChat(chat_id, chats.get(chat_id));
        } catch (SaveErrorException e) {
            e.printStackTrace();
            throw e;
        }
    }

    private String getInformation(int command) {
        switch (command) {
            case COMMAND_START:
                return "Hola. Soy el bot personal de Vylion.";

            case COMMAND_HELP:
                return "Aquí tienes una lista de comandos que acepto:\n\n" +
                        "/start - Repite el saludo inicial\n" +
                        "/help - Saca esta lista\n" +
                        "/pole - Para hacer la pole diaria\n" +
                        "/ranking - Muestra la lista de poles conseguidas.\n" +
                        "\nEa.";

            case WARNING_UNKNOWN_ERROR:
                return "Algo ha ido MUY mal.";

            case WARNING_NOT_IMPLEMENTED:
                return "Aún no está implementado.";

            case WARNING_NO_GROUP:
                return "Debes estar en un grupo para poder hacer eso.";

            case WARNING_NO_USERNAME:
                return "No tienes nombre de usuario. Regístrate con uno en la " +
                        "configuración de Telegram para poder participar.";

            case WARNING_POLE_BLOCKED:
                return "La pole de hoy ya ha sido conseguida. Vuélvelo a intentar mañana.";

            default: return "Entrada incorrecta.";
        }
    }

    private String getFormatExample(int command) {
        switch (command) {
            case COMMAND_START:
                return commands.get(COMMAND_START);

            case COMMAND_HELP:
                return commands.get(COMMAND_HELP);

            case COMMAND_RANKING:
                return commands.get(COMMAND_RANKING);

            case COMMAND_TALIBOT_STATS:
                return commands.get(COMMAND_TALIBOT_STATS);

            default: return "Entrada incorrecta.";
        }
    }
}